@@ -219,7 +219,8 @@ class SpotTheDiffDriftTorch:
         'data' contains the drift prediction, the diffs used to distinguish reference from test instances,
         and optionally the p-value, performance of the classifier relative to its expectation under the
         no-change null, the out-of-fold classifier model prediction probabilities on the reference and test
-        data, and the trained model.
+        data as well as well as the associated reference and test instances of the out-of-fold predictions,
+        and the trained model.
         """
         preds = self._detector.predict(x, return_p_val, return_distance, return_probs, return_model=True)
         preds['data']['diffs'] = preds['data']['model'].diffs.detach().cpu().numpy()  # type: ignore
