@@ -4,9 +4,12 @@ import tensorflow as tf
 from typing import Any, Callable, Optional, Union
 from alibi_detect.cd.base_online import BaseMultiDriftOnline
 from alibi_detect.utils.tensorflow import GaussianRBF, quantile, permed_lsdds
+from alibi_detect.utils.frameworks import Framework
 
 
 class LSDDDriftOnlineTF(BaseMultiDriftOnline):
+    online_state_keys: tuple = ('t', 'test_stats', 'drift_preds', 'test_window', 'k_xtc')
+
     def __init__(
             self,
             x_ref: Union[np.ndarray, list],
@@ -77,7 +80,9 @@ class LSDDDriftOnlineTF(BaseMultiDriftOnline):
             input_shape=input_shape,
             data_type=data_type
         )
-        self.meta.update({'backend': 'tensorflow'})
+        self.backend = Framework.TENSORFLOW.value
+        self.meta.update({'backend': self.backend})
+
         self.n_kernel_centers = n_kernel_centers
         self.lambda_rd_max = lambda_rd_max
 
@@ -96,9 +101,13 @@ class LSDDDriftOnlineTF(BaseMultiDriftOnline):
 
         self._configure_kernel_centers()
         self._configure_thresholds()
-        self._initialise()
+        self._configure_ref_subset()  # self.initialise_state() called inside here
 
     def _configure_normalization(self, eps: float = 1e-12):
+        """
+        Configure the normalization functions used to normalize reference and test data to zero mean and unit variance.
+        The reference data `x_ref` is also normalized here.
+        """
         x_ref_means = tf.reduce_mean(self.x_ref, axis=0)
         x_ref_stds = tf.math.reduce_std(self.x_ref, axis=0)
         self._normalize = lambda x: (x - x_ref_means)/(x_ref_stds + eps)
@@ -117,7 +126,9 @@ class LSDDDriftOnlineTF(BaseMultiDriftOnline):
         self.k_xc = self.kernel(self.x_ref_eff, self.kernel_centers)
 
     def _configure_thresholds(self):
-
+        """
+        Configure the test statistic thresholds via bootstrapping.
+        """
         # Each bootstrap sample splits the reference samples into a sub-reference sample (x)
         # and an extended test window (y). The extended test window will be treated as W overlapping
         # test windows of size W (so 2W-1 test samples in total)
@@ -155,7 +166,20 @@ class LSDDDriftOnlineTF(BaseMultiDriftOnline):
         self.thresholds = thresholds
         self.H_lam_inv = H_lam_inv
 
+    def _initialise_state(self) -> None:
+        """
+        Initialise online state (the stateful attributes updated by `score` and `predict`). This method relies on
+        attributes defined by `_configure_ref_subset`, hence must be called afterwards.
+        """
+        super()._initialise_state()
+        self.test_window = tf.gather(self.x_ref_eff, self.init_test_inds)
+        self.k_xtc = self.kernel(self.test_window, self.kernel_centers)
+
     def _configure_ref_subset(self):
+        """
+        Configure the reference data split. If the randomly selected split causes an initial detection, further splits
+        are attempted.
+        """
         etw_size = 2*self.window_size-1  # etw = extended test window
         nkc_size = self.n - self.n_kernel_centers  # nkc = non-kernel-centers
         rw_size = nkc_size - etw_size  # rw = ref-window
@@ -166,13 +190,20 @@ class LSDDDriftOnlineTF(BaseMultiDriftOnline):
             perm = tf.random.shuffle(tf.range(nkc_size))
             self.ref_inds, self.init_test_inds = perm[:rw_size], perm[-self.window_size:]
             self.c2s = tf.reduce_mean(tf.gather(self.k_xc, self.ref_inds), axis=0)  # (below Eqn 21)
-            self.test_window = tf.gather(self.x_ref_eff, self.init_test_inds)
             # Compute initial lsdd to check for initial detection
-            self.k_xtc = self.kernel(self.test_window, self.kernel_centers)
+            self._initialise_state()  # to set self.test_window and self.k_xtc
             h_init = self.c2s - tf.reduce_mean(self.k_xtc, axis=0)  # (Eqn 21)
             lsdd_init = h_init[None, :] @ self.H_lam_inv @ h_init[:, None]  # (Eqn 11)
 
     def _update_state(self, x_t: tf.Tensor):
+        """
+        Update online state based on the provided test instance.
+
+        Parameters
+        ----------
+        x_t
+            The test instance.
+        """
         self.t += 1
         k_xtc = self.kernel(x_t, self.kernel_centers)
         self.test_window = tf.concat([self.test_window[(1-self.window_size):], x_t], axis=0)
