@@ -1,5 +1,14 @@
 from .missing_optional_dependency import ERROR_TYPES
 from typing import Optional, List, Dict, Iterable
+from enum import Enum
+
+
+class Framework(str, Enum):
+    PYTORCH = 'pytorch'
+    TENSORFLOW = 'tensorflow'
+    KEOPS = 'keops'
+    SKLEARN = 'sklearn'
+
 
 try:
     import tensorflow as tf  # noqa
@@ -14,12 +23,19 @@ try:
 except ImportError:
     has_pytorch = False
 
+try:
+    import pykeops  # noqa
+    import torch  # noqa
+    has_keops = True
+except ImportError:
+    has_keops = False
 
 # Map from backend name to boolean value indicating its presence
 HAS_BACKEND = {
     'tensorflow': has_tensorflow,
     'pytorch': has_pytorch,
-    'sklearn': True
+    'sklearn': True,
+    'keops': has_keops,
 }
 
 
